#pragma once

#include <iostream>
#include <iomanip>

using namespace std;

namespace ADSLibrary
{
	namespace Algorithms
	{
		namespace ArraysAndPointers
		{
			/**
			 * Inicializace hodnot v poli.
			 * Hodnoty v poli jsou pouze demonstran, nemaj dn hlub vznam.
			 *
			 * @param a Pole do kterho se budou ukldat hodnoty
			 * @param N Poet prvk v poli
			 */
			void InitArray(int* a, const int N);

			/**
			 * Zmna hodnot prvk pole.
			 * Zmna hodnot v poli je pouze demonstran.
			 * elem je zmnit hodnoty v poli pi demonstraci koprovn hodnot.
			 *
			 * @param a Pole, kde budeme mnit hodnoty.
			 * @param N Poet prvk v poli
			 */
			void UpdateArray(int* a, const int N);

			/**
			 * Tisk pole na standardn vstup.
			 * Pole je pedvno jako ukazatel, pesnji eeno jako ukazatel na prvn prvek pole.
			 *
			 * @param a Tisknut pole
			 * @param N Poet prvk v poli
			 */
			void PrintArray1(int* a, const int N);

			/**
			 * Tisk pole na standardn vstup.
			 * Pole je pedvno zpsobem obvyklm v C++.
			 *
			 * @param a Tisknut pole
			 * @param N Poet prvk v poli
			 */
			void PrintArray2(int a[], const int N);

			/**
			 * Tisk pole na standardn vstup.
			 * Pole je pedvno jako ukazatel, pesnji eeno jako ukazatel na prvn prvek pole.
			 * Zpracovn pole probh pomoc pointerov aritmetiky.
			 *
			 * @param a Tisknut pole
			 * @param N Poet prvk v poli
			 */
			void PrintArray3(int* a, const int N);

			/**
			 * Koprovn hodnot z pole SourceArray do pole DestinationArray.
			 * Ob pole jsou pedvna jako ukazatel, pesnji eeno jako ukazatel na jejich prvn prvky.
			 * Koprovn probh pomoc bnho indexovn pole.
			 *
			 * @param SourceArray Zdrojov pole, ili "odkud"
			 * @param DestinationArray Clov pole, ili "kam"
			 * @param N Poet prvk v poli SourceArray
			 */
			void CopyArray1(int* SourceArray, int* DestinationArray, const int N);

			/**
			 * Koprovn hodnot z pole SourceArray do pole DestinationArray.
			 * Ob pole jsou pedvna jako ukazatel, pesnji eeno jako ukazatel na jejich prvn prvky.
			 * Koprovn probh pomoc pointerov artimetiky.
			 *
			 * @param SourceArray Zdrojov pole, ili "odkud"
			 * @param DestinationArray Clov pole, ili "kam"
			 * @param N Poet prvk v poli SourceArray
			 */
			void CopyArray2(int* SourceArray, int* DestinationArray, const int N);

			/**
			 * Koprovn hodnot z pole SourceArray do pole DestinationArray.
			 * Ob pole jsou pedvna jako ukazatel, pesnji eeno jako ukazatel na jejich prvn prvky.
			 * Koprovn probh pomoc pointerov aritmetiky.
			 *
			 * Zpis pointerov aritmetiky je v maximln me zhutn,
			 * je vyuito vech dostupnch monost zpisu kdu v C++.
			 * Tento zpsob zpisu algoritmu je zcela nevhodn pro zatenky. Uvdm ho zde dvou dvod -
			 * jednak pro plnost (e je mon i takov zpis algoritmu) a jednak jako hlavolam...
			 *
			 * @param SourceArray Zdrojov pole, ili "odkud"
			 * @param DestinationArray Clov pole, ili "kam"
			 * @param N Poet prvk v poli SourceArray
			 */
			void CopyArray2a(int* SourceArray, int* DestinationArray, const int N);

			/**
			 * Funkce testujc prci s jednorozmrnm polem.
			 */
			void TestArray();

			/**
			 * Poet dk dvourozmrnho pole.
			 */
			const int RowNumber = 4;

			/**
			 * Poet sloupc dvourozmrnho pole.
			 */
			const int ColumnNumber = 5;

			/**
			 * Alokace dvourozmrnho pole. C++ zn jen jednorozmrn pole, dvou a vcerozmrn pole je nutn deklarovat jako pole pointer na dal subpole.
			 * Take dvourozmrn pole dynamicky alokovan je pointer na pole jeho prvky jsou pointery na poadovan datov typ, ili dvojnsobn pointer, **.
			 *
			 * @param R Poet dk alokovanho dvourozmrnho pole
			 * @param C Poet sloupc alokovanho dvourozmrnho pole
			 * @return Pointer na dvourozmrnho pole
			 */
			double** AllocateMatrix(const int R, const int C);

			/**
			 * Dealokace dvourozmrnho pole. Pole je postupn dealokovno po dcch. Dlku dk tohoto pole, tj. poet sloupc, nen nutn uvdt,
			 * protoe dealokace dk matice je provedena pomoc opertoru delete[]. Sprvce pamti v tomto ppad v, kolik kad dek zabr pamti.
			 *
			 * @param M Dynamicky alokovan dvourozmrnho pole
			 * @param R Poet dk dvourozmrnho pole.
			 */
			void DeallocateMatrix(double** M, const int R);

			/**
			 * Naplnn dvourozmrnho pole testovacmi daty.
			 *
			 * @param M Dynamicky alokovan dvourozmrnho pole
			 * @param R Poet dk dvourozmrnho pole
			 * @param C Poet sloupc dvourozmrnho pole
			 */
			void FillMatrix(double** M, const int R, const int C);

			/**
			 * Tisk dvourozmrnho pole na standardn vstup.
			 *
			 * @param M Dynamicky alokovan dvourozmrnho pole
			 * @param R Poet dk dvourozmrnho pole
			 * @param C Poet sloupc dvourozmrnho pole
			 */
			void PrintMatrix(double** M, const int R, const int C);
		}
	}
}
