#pragma once

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace ArrayBasedStructures
		{
			namespace OOP
			{
				/**
				 * \brief Zsobnk implementovan v poli s vyuitm %OOP
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				class Stack
				{
				public:
					/**
					 * Konstruktor. Velikost zsobnku bude nastavena na vchoz velikost.
					 */
					Stack();

					/**
					 * Konstruktor. Velikost zsobnku lze zvolit.
					 *
					 * @param StackSize Velikost zsobnku
					 */
					Stack(const int StackSize);

					/**
					 * Destruktor
					 */
					~Stack();

					/**
					 * Vloen prvku na vrchol zsobnku.
					 *
					 * @param X Vkldan prvek
					 */
					void Push(const char X);

					/**
					 * Odebrn prvku z vrcholu zsobnku.
					 *
					 * @return Prvek z vrcholu zsobnku
					 */
					char Pop();

					/**
					* Prvek na vrcholu zsobnku.
					* Prvek nen ze zsobnku odstrann, jedn se tedy o nedestruktivn variantu funkce #Pop.
					*
					* @return Prvek z vrcholu zsobnku
					*/
					char Top();

					/**
					 * Test, je-li zsobnk przdn.
					 *
					 * @return Funkce vrac true pokud je zsobnk przdn, jinak false.
					 */
					bool IsEmpty() const;

					/**
					 * Test, je-li zsobnk zaplnn.
					 *
					 * @return Funkce vrac true pokud je zsobnk zaplnn, jinak false.
					 */
					bool IsFull() const;

					/**
					 * Smazn celho obsahu zsobnku.
					 */
					void Clear();

				private:
					/**
					 * Pole obsahujc data uloen do zsobnku.
					 */
					char* Items;

					/**
					 * Velikost zsobnku.
					 */
					int StackSize;

					/**
					 * Ukazatel zsobnku (stack pointer).
					 */
					int StackPointer;

					/**
					 * Vchoz (default) velikost zsobnku.
					 */
					static const int DefaultStackSize = 100;
				};
			}
		}
	}
}
