#pragma once

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace ArrayBasedStructures
		{
			namespace Procedural
			{
				/**
				 * Velikost zsobnku.
				 */
				const int StackSize = 100;
				
				/**
				 * \brief Zsobnk implementovan v poli procedurlnm zpsobem.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				struct Stack
				{
					/**
					 * Pole obsahujc poloky zsobnku.
					 */
					char Items[StackSize];
					/**
					 * Ukazatel zsobnku.
					 * Ukazatel zsobnku ukazuje na prvn volnou pozici v poli #Items.
					 */
					int StackPointer;
				};

				/**
				 * Inicializace zsobnku. Zsobnk je po inicializaci przdn.
				 *
				 * @param S Inicializovan zsobnk
				 */
				void Init(Stack& S);

				/**
				 * Vloen prvku na vrchol zsobnku.
				 *
				 * @param S Zsobnk
				 * @param X Vkldan prvek
				 */
				void Push(Stack& S, const char X);

				/**
				 * Odebrn prvku z vrcholu zsobnku.
				 *
				 * @param S Zsobnk
				 * @return Prvek z vrcholu zsobnku
				 */
				char Pop(Stack& S);

				/**
				 * Prvek na vrcholu zsobnku.
				 * Prvek nen ze zsobnku odstrann, jedn se tedy o nedestruktivn variantu funkce #Pop.
				 *
				 * @param S Zsobnk
				 * @return Prvek z vrcholu zsobnku
				 */
				char Top(Stack& S);

				/**
				 * Test, je-li zsobnk przdn.
				 *
				 * @param S Zsobnk
				 * @return Funkce vrac true pokud je zsobnk przdn, jinak false.
				 */
				bool IsEmpty(const Stack& S);

				/**
				 * Test, je-li zsobnk zaplnn.
				 *
				 * @param S Zsobnk
				 * @return Funkce vrac true pokud je zsobnk zaplnn, jinak false.
				 */
				bool IsFull(const Stack& S);

				/**
				 * Smazn celho obsahu zsobnku.
				 *
				 * @param S Zsobnk
				 */
				void Clear(Stack& S);
			}
		}
	}
}