#pragma once

#include <cstdlib>
#include <iostream>
#include <iomanip>

using namespace std;
namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace LinkedStructures
		{
			namespace OOP
			{

				/**
				 * \brief Seznam implementovan pomoc spojovch struktur s vyuitm %OOP
				 *
				 * Implementace obousmrnho seznamu pomoc spojovch struktur.
				 * V seznamu je implementovn jednoduch itertor.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				class List
				{
				public:
					/**
					 * Konstruktor
					 */
					List();

					/**
					 * Destruktor
					 */
					~List();

					/**
					 * Smazn celho seznamu.
					 */
					void Clear();

					/**
					 * Vloen prvku do seznamu. Prvek je vloen ped souasnou hlavu seznamu tj. stane se novou hlavou seznamu.
					 * @param X Vkldan prvek
					 */
					void Insert(const int X);

					/**
					 * Odebrn prvku ze seznamu. Prvek X je nejprve v seznamu nalezen a pot je odpovdajc poloka ze seznamu vymazna.
					 * @param X Odebran prvek
					 */
					void Delete(const int X);

					/**
					 * Hledn prvku v seznamu.
					 * @param X Hledan prvek
					 * @return Funkce vrac true v ppad, e je prvek X nalezen, jinak vrac false.
					 */
					bool Search(const int X);

					/**
					 * Test je-li seznam przdn.
					 * @return Funkce vrac true pokud je seznam przdn, jinak false.
					 */
					bool IsEmpty();

					/**
					 * Funkce vrac poet prvk v seznamu.
					 * @return Poet prvk v seznamu.
					 */
					int Count();

					/**
					 * Reset itertoru. Ukazatel na aktuln poloku seznamu bude ukazovat na hlavu seznamu.
					 */
					void Reset();

					/**
					 * Test ukonen innosti itertoru.
					 * @return Funkce vrac true pokud je iterace seznamem ukonena, jinak false.
					 */
					bool IsEnd();

					/**
					 * Posun ukazatele aktuln poloky na dal poloku v seznamu.
					 */
					void MoveNext();

					/**
					 * Vrcen prvku z aktuln poloky seznamu pi iteraci.
					 * @return Funkce vrac prvek z aktuln poloky seznamu pi iteraci.
					 */
					int Current();

					/**
					 * Pomocn funkce, kter zobrazuje vnitn strukturu seznamu.
					 */
					void Report();

				private:
					/**
					 * \brief Struktura reprezentujc poloku v seznamu.
					 */
					struct ListItem
					{
						/**
						 * Hodnota obsaen v poloce.
						 */
						int Value;
						/**
						 * Ukazatel na pedchoz poloku. Pokud takov poloka neexistuje ukazatel m hodnotu NULL.
						 */
						ListItem* Prev;
						/**
						 * Ukazatel na nsledujc  poloku. Pokud takov poloka neexistuje ukazatel m hodnotu NULL.
						 */
						ListItem* Next;
					};

					/**
					 * Hlava seznamu tj. ukazatel na prvn poloku seznamu.
					 */
					ListItem* mHead;

					/**
					 * Ukazatel na aktuln poloku pi iteraci seznamem.
					 */
					ListItem* mCurrent;
				};
			}
		}
	}
}