#pragma once

#include <cstdlib>
#include <iostream>
#include <iomanip>

using namespace std;

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace LinkedStructures
		{
			namespace OOP
			{
				/**
				 * \brief Seznam implementovan pomoc spojovch struktur s vyuitm %OOP a techniky zarky
				 *
				 * Implementace obousmrnho seznamu pomoc spojovch struktur s vyuitm %OOP a techniky zarky.
				 * V seznamu je implementovn jednoduch itertor.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				class ListWithSentinel
				{
				public:
					/**
					 * Konstruktor
					 */
					ListWithSentinel();

					/**
					 * Destruktor. Krom vlastnho seznamu je z pamti uvolnna i zarka.
					 */
					~ListWithSentinel();

					/**
					 * Smazn celho seznamu, zarka je ponechna v pamti.
					 */
					void Clear();

					/**
					 * Vloen prvku do seznamu. Prvek je vloen ped souasnou hlavu seznamu tj. stane se novou hlavou seznamu.
					 * @param X Vkldan prvek
					 */
					void Insert(const int X);

					/**
					 * Odebrn prvku ze seznamu. Prvek X je nejprve v seznamu nalezen a pot je odpovdajc poloka ze seznamu vymazna.
					 * @param X Odebran prvek
					 */
					void Delete(const int X);

					/**
					 * Hledn prvku v seznamu.
					 * @param X Hledan prvek
					 * @return Funkce vrac true v ppad, e je prvek X nalezen, jinak vrac false.
					 */
					bool Search(const int X);

					/**
					 * Test je-li seznam przdn.
					 * @return Funkce vrac true pokud je seznam przdn, jinak false.
					 */
					bool IsEmpty();

					/**
					 * Funkce vrac poet prvk v seznamu.
					 * @return Poet prvk v seznamu.
					 */
					int Count();

					/**
					 * Reset itertoru. Ukazatel na aktuln poloku seznamu bude ukazovat na hlavu seznamu.
					 */
					void Reset();

					/**
					 * Test ukonen innosti itertoru.
					 * @return Funkce vrac true pokud je iterace seznamem ukonena, jinak false.
					 */
					bool IsEnd();

					/**
					 * Posun ukazatele aktuln poloky na dal poloku v seznamu.
					 */
					void MoveNext();

					/**
					 * Vrcen prvku z aktuln poloky seznamu pi iteraci.
					 * @return Funkce vrac prvek z aktuln poloky seznamu pi iteraci.
					 */
					int Current();

					/**
					 * Pomocn funkce, kter zobrazuje vnitn strukturu seznamu.
					 */
					void Report();

				private:
					/**
					 * \brief Struktura reprezentujc poloku v seznamu.
					 *
					 * Struktura reprezentujc poloku v seznamu.
					 */
					struct ListItem
					{
						/**
						 * Hodnota obsaen v poloce.
						 */
						int Value;
						/**
						 * Ukazatel na pedchoz poloku. Pokud takov poloka neexistuje ukazatel m hodnotu {@link #mNil}.
						 */
						ListItem* Prev;
						/**
						 * Ukazatel na nsledujc  poloku. Pokud takov poloka neexistuje ukazatel m hodnotu {@link #mNil}.
						 */
						ListItem* Next;
					};

					/**
					 * Zarka (sentinel).
					 * Zarka zjednoduuje hranin ppady vkldn, mazn a podobn v seznamu.
					 * Zarka z jinho pohledu pedstavuje vlastn implementaci ukazatele NULL.
					 * Na rozdl od klasickho NULL, se lze jej ptt na hodnotu a nehroz tud havrie programu.
					 * Zarka je souasn hlavou i ocasem seznamu, take seznam je de-facto obousmrn a kruhov.
					 */
					ListItem* mNil;
					
					/**
					 * Ukazatel na aktuln poloku pi iteraci seznamem.
					 */
					ListItem* mCurrent;
				};
			}
		}
	}
}
