#pragma once

#include <iostream>
#include <iomanip>

using namespace std;

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace LinkedStructures
		{
			namespace OOP
			{
				/**
				 * \brief Fronta implementovan pomoc spojovch struktur s vyuitm %OOP.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				class Queue
				{
				public:
					/**
					 * Konstruktor
					 */
					Queue();

					/**
					 * Destuktor
					 */
					~Queue();

					/**
					 * Vloen prvku na konec fronty.
					 *
					 * @param X Vkldan prvek
					 */
					void Enqueue(const int X);

					/**
					 * Odebrn prvku z ela fronty.
					 *
					 * @return Prvek z ela fronty
					 */
					int Dequeue();

					/**
					 * Prvek na ele fronty.
					 * Prvek nen z ela fronty odstrann, jedn se o nedestruktivn variantu metody #Dequeue
					 *
					 * @return Prvek z ela fronty
					 */
					int Peek() const;

					/**
					 * Test, je-li fronta przdn.
					 *
					 * @return Funkce vrac true pokud je fronta przdn, jinak false.
					 */
					bool IsEmpty() const;

					/**
					 * Smazn celho obsahu fronty.
					 *
					 */
					void Clear();

					/**
					 * Pomocn funkce, kter zobrazuje vnitn strukturu fronty.
					 */
					void Report() const;

				private:
					/**
					 * \brief Struktura reprezentujc poloku ve front.
					 */
					struct QueueItem
					{
						/**
						 * Hodnota obsaen v poloce.
						 */
						int Value;
						/**
						 * Ukazatel na pedchoz poloku. Pokud takov poloka neexistuje ukazatel m hodnotu nullptr.
						 */
						QueueItem* Prev;
					};

					/**
					 * Hlava fronty. Hlava fronty ukazuje na prvn prvek fronty.
					 * Prvn prvek fronty nazvme tak elo fronty.
					 */
					QueueItem* Head;

					/**
					 * Ocas fronty. Tento ukazatel bn ukazuje na posledn prvek ve front.
					 */
					QueueItem* Tail;
				};
			}
		}
	}
}
