#pragma once

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace LinkedStructures
		{
			namespace OOPTemplate
			{
				/**
				 * \brief Fronta implementovan pomoc spojovch struktur jako ablona (template) %OOP.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				template <typename T>
				class Queue
				{
				public:
					/**
					 * Konstruktor
					 */
					Queue();

					/**
					 * Destuktor
					 */
					~Queue();

					/**
					 * Vloen prvku na konec fronty.
					 *
					 * @param X Vkldan prvek
					 */
					void Enqueue(const T X);

					/**
					 * Odebrn prvku z ela fronty.
					 *
					 * @return Prvek z ela fronty
					 */
					T Dequeue();

					/**
					 * Prvek na ele fronty.
					 * Prvek nen z ela fronty odstrann, jedn se o nedestruktivn variantu metody #Dequeue
					 *
					 * @return Prvek z ela fronty
					 */
					T Peek() const;

					/**
					 * Test, je-li fronta przdn.
					 *
					 * @return Funkce vrac true pokud je fronta przdn, jinak false.
					 */
					bool IsEmpty() const;

					/**
					 * Smazn celho obsahu fronty.
					 *
					 */
					void Clear();

				private:
					/**
					 * \brief Struktura reprezentujc poloku ve front.
					 */
					struct QueueItem
					{
						/**
						 * Hodnota obsaen v poloce.
						 */
						T Value;
						/**
						 * Ukazatel na pedchoz poloku. Pokud takov poloka neexistuje ukazatel m hodnotu nullptr.
						 */
						QueueItem* Prev;
					};

					/**
					 * Hlava fronty. Hlava fronty ukazuje na prvn prvek fronty.
					 * Prvn prvek fronty nazvme tak elo fronty.
					 */
					QueueItem* Head;

					/**
					 * Ocas fronty. Tento ukazatel bn ukazuje na posledn prvek ve front.
					 */
					QueueItem* Tail;
				};


				template<typename T> Queue<T>::Queue()
				{
					this->Head = nullptr;
					this->Tail = nullptr;
				}

				template<typename T> Queue<T>::~Queue()
				{
					this->Clear();
				}

				template<typename T> void Queue<T>::Enqueue(const T X)
				{
					QueueItem* n = new QueueItem();
					n->Value = X;
					n->Prev = nullptr;
					if (this->IsEmpty())
					{
						this->Head = n;
					}
					else
					{
						this->Tail->Prev = n;
					}
					this->Tail = n;
				}

				template<typename T> T Queue<T>::Dequeue()
				{
					QueueItem* d = this->Head;
					T result = d->Value;
					this->Head = this->Head->Prev;
					delete d;
					return result;
				}

				template<typename T> T Queue<T>::Peek() const
				{
					return this->Head->Value;
				}

				template<typename T> bool Queue<T>::IsEmpty() const
				{
					return this->Head == nullptr;
				}

				template<typename T> void Queue<T>::Clear()
				{
					while (this->IsEmpty())
					{
						this->Dequeue();
					}
					this->Head = nullptr;
					this->Tail = nullptr;
				}
			}
		}
	}
}
