#pragma once

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace LinkedStructures
		{
			namespace OOPTemplate
			{
				/**
				 * \brief Zsobnk implementovan pomoc spojovch struktur jako ablona (template) %OOP.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				template <typename T>
				class Stack
				{
				public:
					/**
					 * Konstruktor.
					 */
					Stack();

					/**
					 * Destruktor.
					 */
					~Stack();

					/**
					 * Vloen prvku na vrchol zsobnku.
					 *
					 * @param X Vkldan prvek
					 */
					void Push(const T X);

					/**
					 * Odebrn prvku z vrcholu zsobnku.
					 *
					 * @return Prvek z vrcholu zsobnku
					 */
					T Pop();

					/**
					 * Prvek na vrcholu zsobnku.
					 * Prvek nen za zsobnku odstrann, jedn se tedy o nedestruktivn variantu funkce #Pop.
					 *
					 * @return Prvek z vrcholu zsobnku
					 */
					T Top() const;

					/**
					 * Test, je-li zsobnk przdn.
					 *
					 * @return Funkce vrac true pokud je zsobnk przdn, jinak false.
					 */
					bool IsEmpty() const;
				
					/**
					 * Smazn celho obsahu zsobnku.
					 */
					void Clear();

				private:
					/**
					 * \brief Struktura reprezentujc poloku v zsobnku.
					 */
					struct StackItem
					{
						/**
						 * Hodnota obsaen v poloce.
						 */
						T Value;
						/**
						 * Ukazatel na pedchoz poloku. Pokud takov poloka neexistuje ukazatel m hodnotu nullptr.
						 */
						StackItem* Prev;
					};

					/**
					 * Ukazatel zsobnku.
					 */
					StackItem* StackPointer;
				};


				template <typename T> Stack<T>::Stack()
				{
					this->StackPointer = nullptr;
				}

				template <typename T> Stack<T>::~Stack()
				{
					Clear();
				}

				template <typename T> void Stack<T>::Push(const T X)
				{
					StackItem* n = new StackItem();
					n->Value = X;
					n->Prev = this->StackPointer;
					this->StackPointer = n;
				}

				template <typename T> T Stack<T>::Pop()
				{
					StackItem* d = this->StackPointer;
					T result = d->Value;
					this->StackPointer = this->StackPointer->Prev;
					delete d;
					return result;
				}

				template <typename T> T Stack<T>::Top() const
				{
					return this->StackPointer->Value;
				}

				template <typename T> bool Stack<T>::IsEmpty() const
				{
					return this->StackPointer == nullptr;
				}

				template <typename T> void Stack<T>::Clear()
				{
					while (!this->IsEmpty())
					{
						this->Pop();
					}
				}
			}
		}
	}
}