#pragma once

#include <iostream>
#include <iomanip>

using namespace std;

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace LinkedStructures
		{
			namespace Procedural
			{
				/**
				 * \brief Struktura reprezentujc poloku ve front {@link #ADSLibrary::DataStructures::LinkedStructures::Procedural::Queue}
				 */
				struct QueueItem
				{
					/**
					 * Hodnota obsaen v poloce.
					 */
					int Value;
					/**
					 * Ukazatel na pedchoz poloku. Pokud takov poloka neexistuje ukazatel m hodnotu nullptr.
					 */
					QueueItem* Prev;
				};

				/**
				 * \brief Fronta implementovan pomoc spojovch struktur procedurlnm zpsobem.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				struct Queue
				{
					/**
					 * Hlava fronty. Hlava fronty ukazuje na prvn prvek fronty.
					 * Prvn prvek fronty nazvme tak elo fronty.
					 */
					QueueItem* Head;
					/**
					 * Ocas fronty. Tento ukazatel bn ukazuje na posledn prvek ve front.
					 */
					QueueItem* Tail;
				};

				/**
				 * Inicializace fronty.
				 *
				 * @param Q Inicializovan fronta
				 */
				void Init(Queue& Q);

				/**
				 * Vloen prvku na konec fronty.
				 *
				 * @param Q Fronta
				 * @param X Vkldan prvek
				 */
				void Enqueue(Queue& Q, const int X);

				/**
				 * Odebrn prvku z ela fronty.
				 *
				 * @param Q Fronta
				 * @return Prvek z ela fronty
				 */
				int Dequeue(Queue& Q);

				/**
				 * Prvek na ele fronty.
				 * Prvek nen z ela fronty odstrann, jedn se o nedestruktivn variantu metody #Dequeue
				 *
				 * @param Q Fronta
				 * @return Prvek z ela fronty
				 */
				int Peek(Queue& Q);

				/**
				 * Test, je-li fronta przdn.
				 *
				 * @param Q Fronta
				 * @return Funkce vrac true pokud je fronta przdn, jinak false.
				 */
				bool IsEmpty(const Queue& Q);

				/**
				 * Smazn celho obsahu fronty.
				 *
				 * @param Q Fronta
				 */
				void Clear(Queue& Q);

				/**
				 * Pomocn funkce, kter zobrazuje vnitn strukturu fronty.
				 *
				 * @param Q Fronta
				 */
				void Report(const Queue& Q);
			}
		}
	}
}