#pragma once

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace LinkedStructures
		{
			namespace Procedural
			{
				/**
				 * \brief Struktura reprezentujc poloku v zsobnku {@link #ADSLibrary::DataStructures::LinkedStructures::Procedural::Stack}
				 */
				struct StackItem
				{
					/**
					 * Hodnota obsaen v poloce.
					 */
					char Value;
					/**
					 * Ukazatel na pedchoz poloku. Pokud takov poloka neexistuje ukazatel m hodnotu nullptr.
					 */
					StackItem* Prev;
				};

				/**
				 * \brief Zsobnk implementovan pomoc spojovch struktur procedurlnm zpsobem.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				struct Stack
				{
					/**
					 * Ukazatel zsobnku
					 */
					StackItem* StackPointer;
				};

				/**
				 * Inicializace zsobnku. Zsobnk je po inicializaci przdn.
				 *
				 * @param S Inicializovan zsobnk
				 */
				void Init(Stack& S);

				/**
				 * Vloen prvku na vrchol zsobnku.
				 *
				 * @param S Zsobnk
				 * @param X Vkldan prvek
				 */
				void Push(Stack& S, const char X);

				/**
				 * Odebrn prvku z vrcholu zsobnku.
				 *
				 * @param S Zsobnk
				 * @return Prvek z vrcholu zsobnku
				 */
				char Pop(Stack& S);

				/**
				 * Prvek na vrcholu zsobnku.
				 * Prvek nen ze zsobnku odstrann, jedn se tedy o nedestruktivn variantu funkce #Pop.
				 *
				 * @param S Zsobnk
				 * @return Prvek z vrcholu zsobnku
				 */
				char Top(Stack& S);

				/**
				 * Test, je-li zsobnk przdn.
				 *
				 * @param S Zsobnk
				 * @return Funkce vrac true pokud je zsobnk przdn, jinak false.
				 */
				bool IsEmpty(const Stack& S);

				/**
				 * Smazn celho obsahu zsobnku.
				 *
				 * @param S Zsobnk
				 */
				void Clear(Stack& S);
			}
		}
	}
}